;;; Model code for interval estimation and bisection experiments from:
;;;
;;; N.A. Taatgen, H. van Rijn, J.R. Anderson. An Integrated theory of prospective time estimation:
;;; The Role of Cognition, Attention and Learning
;;;
;;; To run the interval estimation model, use:
;;; (do-all)
;;; to run all three intervals, or
;;; (do-experiments t train n m)
;;; to run an interval experiment with interval t (seconds), train training trials, n virtual subjects
;;; and m tests per subject
;;; The output is a list of the estimates the model makes, which can be fed into any analysis program
;;; to create a histogram.
;;;
;;; To run the bisection experiment, use one of the following:
;;; For the 3/6 second version:
;;; (do-disc-experiment n 3.0 6.0 *test1*)
;;; with n the number of virtual subjects
;;; For the 4/12 second version:
;;; (do-disc-experiment n 4.0 12.0 *test2*)
;;; For the 2/8 second version:
;;; (do-disc-experiment n 2.0 8.0 *test3*)
;;;
;;; The output consists of a list of probe times followed by 0 or 1. A 0 indicates the model thought
;;; the interval was short, and a 1 indicates the model thought the interval was long.
;;; A pivottable in Excel or something similar can then be used to calculate the averages for each interval



(defvar *time-noise*)
(defvar *time-mult*)
(defvar *start-increment*)
(defvar *master-start-increment*)
(defvar *temporal*)

(setf *time-noise* 0.015 *time-mult* 1.1 *master-start-increment* .011)

;;; The following production can be used to simulate the behavior
;;; of the temporal buffer using lisp functions
;;; They are not necessary for the buffer itself

(defun noise (s)
  "Approximates a sample from a normal distribution with mean zero and
   the given s-value (/ (sqrt (* 3.0 variance)) 3.1416)."
  ;; Need to test bound because of short-float lack of precision
  (let ((p (max 0.0001 (min (random 1.0) 0.9999))))
    (* s (log (/ (- 1.0 p) p)))))
(defun start-subject ()
  (setf *start-increment* (+ *master-start-increment* (noise (* *master-start-increment* *time-noise* 5)))))

(start-subject)

(defun test-run (n)
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0))
    (dotimes (j n)
      (format t "~4D ~8,4F ~8,4F~%" j i sum)
      (setf sum (+ sum i))
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i)))))))

(defun test-criterium (tt)
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0)
        (j 0))
    (loop until (> sum tt) do
;      (format t "~4D ~8,4F ~8,4F~%" j i sum)
      (setf sum (+ sum i))
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i))))
      (setf j (1+ j)))
    j))

(defun find-cycles (tt n)
  (let ((sum 0.0))
    (dotimes (i n) (setf sum (+ sum (test-criterium tt))))
    (/ sum n)))

(defun find-time (m n)
  (let ((sum 0.0))
    (dotimes (i n) (setf sum (+ sum (run-timer m))))
    (/ sum n)))

(defun run-timer (n)
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0))
    (dotimes (j n)
;      (format t "~4D ~8,4F ~8,4F~%" j i sum)
      (setf sum (+ sum i))
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i)))))
    sum))

(defun run-multi (m n)
  (dotimes (i n)
    (format t "~6,4F~%" (run-timer m))))

(defun do-experiments (tt train n m)
  (dotimes (i n)
    (start-subject)
    (let ((cycles (truncate (find-cycles tt train))))
      (run-multi cycles m))))

(defun do-counting (tt train n m)
  (dotimes (i n)
    (start-subject)
    (let* ((cycles (truncate (find-cycles 1.0 train)))
           (metacycles (round (/ tt (find-time cycles train)))))
      (dotimes (j m)
        (let ((sum 0.0))
          (dotimes (k metacycles)
            (setf sum (+ sum (run-timer cycles))))
          (format t "~8,4F~%" sum))))))

(defun do-all ()
  (format t "~%**** 8 seconds~%")
  (do-experiments 8.0 5 200 80)

  (format t "~%~%**** 12 seconds~%")
  (do-experiments 12.0 5 200 80)

  (format t "~%~%**** 21 seconds~%")
  (do-experiments 21.0 5 200 80)
)

(defun do-rosenbaum (n)
  (let ((short (truncate (find-cycles .30 1000)))
        (middle (truncate  (find-cycles .417 1000)))
        (long (truncate (find-cycles .65 1000)))
        (sum 0)
        (reference-time .37))
    ;;; first the lengthened trials
    (dotimes (i n)
      (let* ((wait (run-timer short))
             (rt (+ reference-time (if (< wait .417) 0.0
                                       (if (> wait .467) .05
                                           (- .467 wait))))))
        (setf sum (+ sum rt))))
        (format t "~%Short  ~6,4F" (/ sum n))
        (setf sum 0)
    (dotimes (i n)
      (let* ((wait (run-timer middle))
             (rt (+ reference-time (if (< wait .417) 0.0
                                       (if (> wait .467) .05
                                           (- .467 wait))))))
        (setf sum (+ sum rt))))
        (format t "~%Middle ~6,4F" (/ sum n))
         (setf sum 0)
    (dotimes (i n)
      (let* ((wait (run-timer long))
             (rt (+ reference-time (if (< wait .417) 0.0
                                       (if (> wait .467) .05
                                           (- .467 wait))))))
        (setf sum (+ sum rt))))
        (format t "~%Long  ~6,4F" (/ sum n))
       

))

(defun do-oscillator (n)
  (dotimes (i 35)
    (let* ((tt (+ .15 (* i .025)))
           (cycles (truncate (find-cycles tt 10)))
           (sum 0.0))
      (dotimes (j n)
        (setf sum (+ sum (run-timer cycles))))
      (format t "~% ~6,3F  ~6,3F" tt (/ sum (* n tt))))))
      
;;; Code for discrimitation experiment

(defparameter *test1* '(3.0 3.37 3.78 4.24 4.76 5.34 6.0))
(defparameter *test2* '(4.0 4.8 5.77 6.93 8.32 9.99 12.0))
(defparameter *test3* '(2.0 2.52 3.18 4.0 5.04 6.35 8.0))

(defun do-disc-experiment (n short long test-list)
;;; training phase
  (dotimes (i n)
    (let ((short-no (find-cycles short 5))
          (long-no (find-cycles long 5)))
      (start-subject)
      (dotimes (j 100)
        (let* ((stim (nth (random (length test-list)) test-list))
               (cycles (find-cycles stim 1))
               (choice (cond ((> (- long-no cycles) (- cycles short-no)) 0)
                             ((< (- long-no cycles) (- cycles short-no)) 1)
                             (t (random 2)))))
          (format t "~5,2F ~1D~%" stim choice))))))
                

