;;; Model code for the Zakay experiment from:
;;;
;;; N.A. Taatgen, H. van Rijn, J.R. Anderson. An Integrated theory of prospective time estimation:
;;; The Role of Cognition, Attention and Learning
;;;
;;; To run the experiment, type (do-all n), with n the number of virtual subjects
;;; 1000 subjects is a good number
;;;

(defvar *time-noise*)
(defvar *time-mult*)
(defvar *start-increment*)
(defvar *master-start-increment*)
(defvar *temporal*)

(setf *time-noise* 0.015 *time-mult* 1.1 *master-start-increment* .011)

;;; The following production can be used to simulate the behavior
;;; of the temporal buffer using lisp functions
;;; They are not necessary for the buffer itself
(defun noise (s)
  "Approximates a sample from a normal distribution with mean zero and
   the given s-value (/ (sqrt (* 3.0 variance)) 3.1416)."
  ;; Need to test bound because of short-float lack of precision
  (let ((p (max 0.0001 (min (random 1.0) 0.9999))))
    (* s (log (/ (- 1.0 p) p)))))
(defun start-subject ()
  (setf *start-increment* (+ *master-start-increment* (noise (* *master-start-increment* *time-noise* 5)))))

(start-subject)

(defun test-run (n )
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0)
        (sec 0))
    (dotimes (j n)
      (format t "~4D ~8,4F ~8,4F~%" j i sum)
      (setf sum (+ sum i))
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i)))))))

(defun test-criterium (tt &key (disrupt 0.0))
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0)
        (j 0)
        (sec 0))
    (loop until (> sum tt) do
      (setf sum (+ sum i))
      (loop until (<= sum sec) do
        (incf sec)
        (when (< (random 1.0) disrupt) 
          (setf j 0 i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
          )
       )
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i))))
      (setf j (1+ j)))
    j
))

(defun find-cycles (tt n)
  (let ((sum 0.0))
    (dotimes (i n) (setf sum (+ sum (test-criterium tt))))
    (/ sum n)))

(defun find-time (m n)
  (let ((sum 0.0))
    (dotimes (i n) (setf sum (+ sum (run-timer m))))
    (/ sum n)))

(defun run-timer (n &key (disrupt 0.0))
  (let ((i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
        (sum 0.0)        
        (sec 0)
        (j 0)
        )
    (loop until (= j n) do
      (setf sum (+ sum i))
      (setf i (* i *time-mult*))
      (setf i (+ i (noise (* *time-noise* i))))
      (incf j)
      (loop until (<= sum sec) do
        (incf sec)
        (when (< (random 1.0) disrupt) 
          (setf j 0 
                i (+ *start-increment* (noise (* *time-noise* *start-increment*))))
          )
       )
  )
    sum))

(defun run-multi (m n)
  (dotimes (i n)
    (format t "~6,4F~%" (run-timer m))))

(defun do-experiments (tt train n m)
  (dotimes (i n)
    (start-subject)
    (let ((cycles (truncate (find-cycles tt train))))
      (run-multi cycles m))))

(defun do-counting (tt train n m)
  (dotimes (i n)
    (start-subject)
    (let* ((cycles (truncate (find-cycles 1.0 train)))
           (metacycles (round (/ tt (find-time cycles train)))))
      (dotimes (j m)
        (let ((sum 0.0))
          (dotimes (k metacycles)
            (setf sum (+ sum (run-timer cycles))))
          (format t "~8,4F~%" sum))))))

(defun zakay (n tt dr1 dr2)
  (let ((accum nil))
    (dotimes (i n)
      (start-subject)
      (let ((x (1- (test-criterium tt :disrupt dr1))))
        (push (run-timer x :disrupt dr2) accum)))
        (let* ((avg (/ (apply #'+ accum) (length accum)))
               (dev (sqrt (/ (apply #'+ (mapcar #'(lambda (y) (* (- y avg) (- y avg))) accum)) (length accum)))))
          (format t "AVG ~6,3F  SD ~6,3F~%" avg dev))))


(defun do-all (n)
  (format t "Results of Zakay model~%")
  (format t "Perceive vs. Reproduce~%")
  (format t "ET  vs. ET  ")
  (zakay n 12.0 0.0 0.0)
  (format t "ET  vs. W   ")
  (zakay n 12.0 0.0 0.01)
  (format t "ET  vs. CW  ")
  (zakay n 12.0 0.0 0.035)
  (format t "ET  vs. CWA ")
  (zakay n 12.0 0.0 0.055)
  (format t "W   vs. ET  ")
  (zakay n 12.0 0.01 0.0)
  (format t "CW  vs. ET  ")
  (zakay n 12.0 0.035 0.0)
  (format t "CWA vs. ET  ")
  (zakay n 12.0 0.055 0.0))
  
